﻿#include "stdafx.h"
#include "SettingFrame.hpp"
#include "NeMessengerDlg.h"
#include "DuiDefine.h"
#include "..\..\DuiLib\Utils\WinImplBase.h"
#include "NMessageBox.h"
#include "SettingGeneral.h"
#include "SettingPersonal.h"
#include "SettingMessage.h"
#include "SettingAutoText.h"
#include "AutoTextListElementUI.h"

#define SETTING_FRAME_WIDTH						445
#define SETTING_FRAME_HEIGHT					422
#define SETTING_FRAME_FLASH						1
#define SETTING_FRAME_FLASH_TIME				100

#define kSettingAutoText_Setting_Add			_T("setting_add")
#define kSettingAutoText_Setting_Save			_T("setting_save")
#define kSettingAutoText_Setting_Delete			_T("setting_delete")
#define kSettingAutoText_Setting_ListView		_T("setting_autoText_listview")
#define kSettingAutoText_Setting_cb_Shortcut	_T("setting_cb_shortcut")
#define kSettingAutoText_Setting_RichEdit		_T("setting_rich_edit")

const TCHAR* const kSettingLabArrow[4] = {_T("arrow_general"), _T("arrow_personal"), _T("arrow_message"), _T("arrow_autoText")};
const TCHAR* const kSettingButton[4] = {_T("opt_general"), _T("opt_personal"), _T("opt_message"), _T("opt_autoText")};

DUI_BEGIN_MESSAGE_MAP(CSettingFrame, WindowImplBase, FALSE)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_WINDOWINIT, OnPrepare)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_CLICK, NotifyCtrlClick)     //kClick
    DUI_ON_MSGTYPE(DUI_MSGTYPE_SELECTCHANGED, NotifyTabSelectChange)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_TEXTCHANGED, NotifyTextChange)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_ITEMSELECT, NotifyItemSelect)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_SETFOCUS, OnNotifySetFocus)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_KILLFOCUS, OnNotifyKillFocus)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_TIMER, NotifyCtrlTime)
	DUI_ON_MSGTYPE(DUI_MSGTYPE_ITEMCLICK_AUTOTEXT, NotifyItemClick)
	DUI_ON_MSGTYPE(DUI_MSGTYPE_ITEMDBCLICK, OnNotifyItemDoubleClick)
DUI_END_MESSAGE_MAP()

DUI_BEGIN_CUSTOM_MSG_MAP(CSettingFrame)
    ON_CUSTOM_MSG(WM_KEYDOWN, _OnMsgKeyDown)
    ON_CUSTOM_MSG(WM_WINDOWPOSCHANGING, _OnMsgWindowPosChanging)
DUI_END_CUSTOM_MSG_MAP()

CSettingFrame::CSettingFrame()
{
	m_nCurSelTab = TAB_SETTING_GENERAL;
	m_pSettingPersonal = NULL;
	m_pSettingGeneral = NULL;
	m_pSettingMessage = NULL;
	m_pSettingAutoText = NULL;
    m_nFlashTimes = 0;
    m_bClosing = FALSE;
    m_bEnableFlash = FALSE;
}

CSettingFrame::~CSettingFrame()
{

}

void CSettingFrame::InitWindow()
{
	CLabelUI* titleName = static_cast<CLabelUI*>(m_PaintManager.FindControl(kAddChatTitleName));
	if (titleName != NULL)
	{
		titleName->SetText(LoadStringFromResource(IDS_STRING530));
	}
	
	CButtonUI* pBtnOK = static_cast<CButtonUI*>(m_PaintManager.FindControl(kSettingOK));
	if (pBtnOK != NULL)
	{
		pBtnOK->SetText(LoadStringFromResource(IDS_STRING568));
	}
	CButtonUI* pBtnCancel = static_cast<CButtonUI*>(m_PaintManager.FindControl(kSettingCancel));
	if (pBtnCancel != NULL)
	{
		pBtnCancel->SetText(LoadStringFromResource(IDS_STRING569));
	}

	CButtonUI* pBtnAdd = static_cast<CButtonUI*>(m_PaintManager.FindControl(kSettingAdd));
	if (pBtnAdd != NULL)
	{
		pBtnAdd->SetText(LoadStringFromResource(IDS_STRING_ADD));
	}

	CButtonUI* pBtnSave = static_cast<CButtonUI*>(m_PaintManager.FindControl(kSettingSave));
	if (pBtnSave != NULL)
	{
		pBtnSave->SetText(LoadStringFromResource(IDS_STRING_SAVE));
	}

	CButtonUI* pBtnDelete = static_cast<CButtonUI*>(m_PaintManager.FindControl(kSettingDelete));
	if (pBtnDelete != NULL)
	{
		pBtnDelete->SetText(LoadStringFromResource(IDS_STRING_DELETE));
	}

	COptionUI* pBtnGeneral = static_cast<COptionUI*>(m_PaintManager.FindControl(kSettingOptGeneral));
	if (pBtnGeneral != NULL)
	{
		pBtnGeneral->SetText(LoadStringFromResource(IDS_STRING412));
	}
	COptionUI* pBtnPersonal = static_cast<COptionUI*>(m_PaintManager.FindControl(kSettingOptPersonal));
	if (pBtnPersonal != NULL)
	{
		pBtnPersonal->SetText(LoadStringFromResource(IDS_STRING413));
	}
	COptionUI* pBtnMessage = static_cast<COptionUI*>(m_PaintManager.FindControl(kSettingOptMessage));
	if (pBtnMessage != NULL)
	{
		pBtnMessage->SetText(LoadStringFromResource(IDS_STRING414));
	}
	COptionUI* pBtnAutoText = static_cast<COptionUI*>(m_PaintManager.FindControl(kSettingOptAutoText));
	if (pBtnAutoText != NULL)
	{
		pBtnAutoText->SetText(LoadStringFromResource(IDS_STRING393));
	}
	CButtonUI* pBtnClose = static_cast<CButtonUI*>(m_PaintManager.FindControl(kCloseButtonControlName));
	if (pBtnClose != NULL)
	{
		pBtnClose->SetToolTip(LoadStringFromResource(IDS_STRING_CHAT_TOOLTIP_7));
	}

	m_pSettingGeneral = static_cast<CSettingGeneral*>(m_PaintManager.FindControl(kSettingTabGeneral));
	ASSERT(m_pSettingGeneral != NULL);
	m_pSettingGeneral->InitControl();

	m_pSettingPersonal = static_cast<CSettingPersonal*>(m_PaintManager.FindControl(kSettingTabPersonal));
	ASSERT(m_pSettingPersonal != NULL);
	m_pSettingPersonal->InitControl();

	m_pSettingMessage = static_cast<CSettingMessage*>(m_PaintManager.FindControl(kSettingTabMessage));
	ASSERT(m_pSettingMessage != NULL);
	m_pSettingMessage->InitControl();

	m_pSettingAutoText = static_cast<CSettingAutoText*>(m_PaintManager.FindControl(kSettingTabAutoText));
	ASSERT(m_pSettingAutoText != NULL);
	m_pSettingAutoText->InitControl();
	
	m_bEnableFlash = TRUE;
	m_nFlashTimes = 0;
}

void CSettingFrame::SetWindowPosition(CRect rcParent)
{
    CRect rcMonitor(0, 0, 0, 0);
	MONITORINFO oMonitor = {};
	oMonitor.cbSize = sizeof(oMonitor);
	::GetMonitorInfo(::MonitorFromWindow(GET_MESSENGER_DLG()->GetSafeHwnd(), MONITOR_DEFAULTTONEAREST), &oMonitor);
	rcMonitor = oMonitor.rcWork;
	int nLeft = rcParent.left - SETTING_FRAME_WIDTH - 5;
	int nRight = rcParent.right + SETTING_FRAME_WIDTH + 5;
	if (nLeft<rcMonitor.left && nRight>rcMonitor.right)
	{
		CenterWindow();
		ShowWindow(true, true);
		return;
	}
	if(nLeft < rcMonitor.left)
	{
		nLeft = rcParent.right + 5;
	}
	::SetWindowPos(m_hWnd, NULL, nLeft, rcParent.top, SETTING_FRAME_WIDTH, SETTING_FRAME_WIDTH, SWP_FRAMECHANGED | SWP_SHOWWINDOW);
}

LPCTSTR CSettingFrame::GetWindowClassName() const
{
	return _T("SettingWindow");
}

void CSettingFrame::OnFinalMessage(HWND hWnd)
{
	WindowImplBase::OnFinalMessage(hWnd);
	::PostMessage(GET_MESSENGER_DLG()->GetSafeHwnd(), MSG_CLOSE_SETTING_FRAME, 0, 0);
	delete this;
}


void CSettingFrame::NotifyCtrlTime(TNotifyUI& msg)
{
	if (_tcsicmp(msg.pSender->GetName(), kAddChatTitleName) == 0)
	{
		CLabelUI* titleName = static_cast<CLabelUI*>(m_PaintManager.FindControl(kAddChatTitleName));
		if (titleName != NULL)
		{
			if (m_nFlashTimes > 10)
			{
				m_PaintManager.KillTimer(titleName);
				titleName->SetText(_T(""));
				m_nFlashTimes = 0;
			}
			CString strText = titleName->GetText();
			if (strText.GetLength() > 0)
			{
				titleName->SetText(_T(""));
			}
			else
			{
				titleName->SetText(LoadStringFromResource(IDS_STRING530));
			}
			m_nFlashTimes++;
		}
	}
}

void CSettingFrame::NotifyItemClick(TNotifyUI& msg)
{
	if (m_pSettingAutoText)
	{
		m_pSettingAutoText->NotifyItemClick(msg);
	}
}

void CSettingFrame::SaveAllConfig()
{
	if (!m_pSettingPersonal->DoApply())
	{
		// OnBnClickedConfigPersonalButton();
		return;
	}

	if (!m_pSettingGeneral->DoApply())
	{
		return;
	}
	if (!m_pSettingMessage->DoApply())
	{
		return;
	}

	GET_CONFIG()->SaveClientConfig();
	if(GET_CONFIG()->GetAutoStart() == 1)
	{
		GET_CONFIG()->AppAutoStart(TRUE);
	}
	else
	{
		GET_CONFIG()->AppAutoStart(FALSE);

	}

	GET_MESSENGER_DLG()->ProcRenameTalkname(GET_CONFIG()->GetChangeNickName());

	GET_MESSENGER_DLG()->CloseModalDlg(TRUE);
	CloseSettingFrame();
}
void CSettingFrame::NotifyCtrlClick(TNotifyUI& msg)
{
	if (_tcsicmp(msg.pSender->GetName(), kCloseButtonControlName) == 0)
	{
		//m_pSettingAutoText->Save();
		CloseSettingFrame();
	}
	else if (_tcsicmp(msg.pSender->GetName(), kSettingOK) == 0)
	{
		SaveAllConfig();
	}
	else if (_tcsicmp(msg.pSender->GetName(), kSettingCancel) == 0)
	{
		CloseSettingFrame();
	}
	else if (msg.pSender->GetName() == kSettingAutoText_Setting_Save
		|| msg.pSender->GetName() == kSettingAutoText_Setting_Delete
		|| msg.pSender->GetName() == kSettingAutoText_Setting_Add)
	{
	//	m_pSettingAutoText->NotifyCtrlClick(msg);
	}
	else
	{
		switch (m_nCurSelTab)
		{
		case TAB_SETTING_MESSAGE:
			if(m_pSettingMessage)
				m_pSettingMessage->NotifyCtrlClick(msg);
			break;
		case TAB_SETTING_GENERAL:
			if(m_pSettingGeneral)
				m_pSettingGeneral->NotifyCtrlClick(msg);
			break;
		case TAB_SETTING_PERSONAL:
			if(m_pSettingPersonal)
				m_pSettingPersonal->NotifyCtrlClick(msg);
			break;
		case TAB_SETTING_AUTOTEXT:
			break;
		}
		
		
	}
}

void CSettingFrame::NotifyTabSelectChange(TNotifyUI& msg)
{
	CTabLayoutUI* pTabControl = static_cast<CTabLayoutUI*>(m_PaintManager.FindControl(kSettingTabs));
	if (_tcsicmp(msg.pSender->GetName(), kSettingOptGeneral) == 0)
	{
		if (pTabControl && pTabControl->GetCurSel() != TAB_SETTING_GENERAL)
		{
			pTabControl->SelectItem(TAB_SETTING_GENERAL);
			SetSelectButton(false);
			m_nCurSelTab = TAB_SETTING_GENERAL;
			SetSelectButton(true);
			//SwitchFrameButton(false);
		}
	}
	else if (_tcsicmp(msg.pSender->GetName(), kSettingOptPersonal) == 0)
	{
		if (pTabControl && pTabControl->GetCurSel() != TAB_SETTING_PERSONAL)
		{
			pTabControl->SelectItem(TAB_SETTING_PERSONAL);
			SetSelectButton(false);
			m_nCurSelTab = TAB_SETTING_PERSONAL;
			SetSelectButton(true);
			//SwitchFrameButton(false);
			m_PaintManager.SetFocus(NULL);
		}
	}
	else if (_tcsicmp(msg.pSender->GetName(), kSettingOptMessage) == 0)
	{
		if (pTabControl && pTabControl->GetCurSel() != TAB_SETTING_MESSAGE)
		{
			pTabControl->SelectItem(TAB_SETTING_MESSAGE);
			SetSelectButton(false);
			m_nCurSelTab = TAB_SETTING_MESSAGE;
			SetSelectButton(true);
			//SwitchFrameButton(false);
		}
	}
	else if (_tcsicmp(msg.pSender->GetName(), kSettingOptAutoText) == 0)
	{
		if (pTabControl && pTabControl->GetCurSel() != TAB_SETTING_AUTOTEXT)
		{
			pTabControl->SelectItem(TAB_SETTING_AUTOTEXT);
			SetSelectButton(false);
			m_nCurSelTab = TAB_SETTING_AUTOTEXT;
			SetSelectButton(true);
			//SwitchFrameButton(true);
		}
	}
	else if (_tcsicmp(msg.pSender->GetName(), kSettingPersonalExportGroup) == 0)
	{
		m_pSettingPersonal->SetGroupType(PERSONAL_GROUP_EXPORT);
	}
	else if (_tcsicmp(msg.pSender->GetName(), kSettingPersonalImportGroup) == 0)
	{
		m_pSettingPersonal->SetGroupType(PERSONAL_GROUP_IMPORT);
	}
}

void CSettingFrame::NotifyTextChange(TNotifyUI& msg)
{

	if (msg.pSender->GetName() == kSettingAutoText_Setting_RichEdit)
	{
		//m_pSettingAutoText->NotifyTextChange(msg);
	}
	else
	{
		m_pSettingPersonal->NotifyTextChange(msg);
	}
}

void CSettingFrame::OnNotifyItemDoubleClick(TNotifyUI& msg)
{
	if (m_pSettingPersonal != NULL)
	{
		m_pSettingPersonal->OnNotifyItemDoubleClick(msg);
	}
}

void CSettingFrame::NotifyItemSelect(TNotifyUI& msg)
{
	if (msg.pSender->GetName() == kSettingAutoText_Setting_ListView
		|| msg.pSender->GetName() == kSettingAutoText_Setting_cb_Shortcut)
	{
		if (m_pSettingAutoText != NULL)
		{
			//m_pSettingAutoText->NotifyItemSelect(msg);
		}
	}
	else if (msg.pSender->GetName() == kSettingPersonalTimeCombo)
	{
		if (m_pSettingPersonal != NULL)
		{
			m_pSettingPersonal->NotifyItemSelect(msg);
		}
		
	}
	else if (msg.pSender->GetName() == kSettingGeneralLanCombo || msg.pSender->GetName() == kSettingGeneralTimeZoneCombo)
	{
		if (m_pSettingGeneral != NULL)
		{
			m_pSettingGeneral->NotifyItemSelect(msg);
		}

	}
	
}

void CSettingFrame::OnNotifySetFocus(TNotifyUI& msg)
{
    //m_pSettingAutoText->NotifyCtrlFocus(msg);
}

void CSettingFrame::OnNotifyKillFocus(TNotifyUI& msg)
{
    //m_pSettingAutoText->NotifyCtrlKillFocus(msg);
}

void CSettingFrame::SetSelectButton(bool bSelect)
{
	CLabelUI* pLab = static_cast<CLabelUI*>(m_PaintManager.FindControl(kSettingLabArrow[m_nCurSelTab]));
	if (pLab != NULL)
	{
		pLab->SetVisible(bSelect);
	}
	COptionUI* pBtn = static_cast<COptionUI*>(m_PaintManager.FindControl(kSettingButton[m_nCurSelTab]));
	if (pBtn != NULL)
	{
		if (bSelect)
		{
			pBtn->SetFont(2);
		}
		else
		{
			pBtn->SetFont(1);
		}
	}
}

// void CSettingFrame::SwitchFrameButton(bool bSwitch)
// {
// 	CHorizontalLayoutUI* pLayout = static_cast<CHorizontalLayoutUI*>(m_PaintManager.FindControl(_T("tab_2_btn")));
// 	if (pLayout != NULL)
// 	{
// 		pLayout->SetVisible(!bSwitch);
// 	}
// 	pLayout = static_cast<CHorizontalLayoutUI*>(m_PaintManager.FindControl(_T("tab_3_btn")));
// 	if (pLayout != NULL)
// 	{
// 		pLayout->SetVisible(bSwitch);
// 	}
//}
void CSettingFrame::SetEnableFlash(BOOL bEnableFlash)
{
	m_bEnableFlash = bEnableFlash;
}

LRESULT CSettingFrame::_OnMsgKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    bHandled = FALSE;
    if ((TCHAR)wParam == VK_ESCAPE)
    {
        CloseSettingFrame();
        bHandled = TRUE;
    }
    return 0;
}

LRESULT CSettingFrame::_OnMsgWindowPosChanging(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    bHandled = FALSE;
    WINDOWPOS wp = *(LPWINDOWPOS)lParam;
    UINT flags = SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOMOVE;
    CPoint pt(0, 0);
    GetCursorPos(&pt);
    CRect rcMainWnd(0, 0, 0, 0);
	HWND hMainFrame = GET_MESSENGER_DLG()->GetSafeHwnd();
	HWND hWndFocus = ::GetForegroundWindow();
	CRect rcWnd;
	GetWindowRect(m_hWnd, rcWnd);
	 GetWindowRect(hMainFrame, rcMainWnd);
    if ((wp.flags == flags) &&  !rcWnd.PtInRect(pt) && rcMainWnd.PtInRect(pt) && (m_hWnd == hWndFocus))
    {
        if (m_bEnableFlash)
        {
            FlashSettingFrame();
            bHandled = TRUE;
			return 0;
        }
    }
    bHandled = FALSE;
    return 0;
}


void CSettingFrame::FlashSettingFrame()
{
	CLabelUI* titleName = static_cast<CLabelUI*>(m_PaintManager.FindControl(kAddChatTitleName));
	m_PaintManager.SetTimer(titleName, SETTING_FRAME_FLASH, SETTING_FRAME_FLASH_TIME);
}

void CSettingFrame::CloseSettingFrame()
{
	m_bEnableFlash = FALSE;
	Close();
}

void CSettingFrame::OnPrepare(TNotifyUI& msg)
{
	m_nCurSelTab= 0;
	/*
	CHorizontalLayoutUI* tabbar = static_cast<CHorizontalLayoutUI*>(m_PaintManager.FindControl(_T("setting_tabbar")));
	CDialogBuilder Builder;
	CVerticalLayoutUI* pVetticalLayout = static_cast<CVerticalLayoutUI*>(Builder.Create(_T("setting_tabbar.xml"), (UINT)0, NULL, &m_PaintManager));
	tabbar->Add(pVetticalLayout);
	*/
}

CControlUI* CSettingFrame::CreateControl(LPCTSTR pstrClass)
{
	if (_tcsicmp(pstrClass, _T("SettingGeneral")) == 0)
	{
		return new CSettingGeneral();
	}
	if (_tcsicmp(pstrClass, _T("SettingPersonal")) == 0)
	{
		return new CSettingPersonal(this, m_PaintManager);
	}
	if (_tcsicmp(pstrClass, _T("SettingMessage")) == 0)
	{
		return new CSettingMessage(m_PaintManager);
	}
	if (_tcsicmp(pstrClass, _T("SettingAutoText")) == 0)
	{
		return new CSettingAutoText(this);
	}

	return NULL;
}

CDuiString CSettingFrame::GetSkinFile()
{
	return _T("setting_frame.xml");
}

CDuiString CSettingFrame::GetSkinFolder()
{
	return _T("skin\\nTalkRes");
}

LRESULT CSettingFrame::OnKillFocus(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	return 0;
}

LRESULT CSettingFrame::OnNcHitTest(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	int ret = __super::OnNcHitTest(uMsg, wParam, lParam, bHandled);
	if (ret != HTCAPTION)
	{
		ret = HTCLIENT;
	}
	return ret;
}







